/* OGMDvd - A helper library against libdvdread
 * Copyright (C) 2004-2005 Olivier Rolland <billl@esil.univ-mrs.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "config.h"
#include "ogmdvd.h"

#include <stdio.h>
#include <string.h>

#include <dvdread/ifo_types.h>
#include <dvdread/ifo_read.h>

struct _OGMDvd
{
  guint ref;
  gchar *device;
  dvd_reader_t *reader;
  ifo_handle_t *vmg_file;
};

struct _OGMDvdVideoStream
{
  OGMDvd *dvd;

  guint ref;
  guint vid;
  guint ttn;
  guint pgcn;
  ifo_handle_t *vts_file;
};

struct _OGMDvdAudioStream
{
  OGMDvdVideoStream *video;

  guint ref;
  guint aid;
  audio_attr_t *attr;
};

struct _OGMDvdSubpStream
{
  OGMDvdVideoStream *video;

  guint ref;
  guint sid;
  subp_attr_t *attr;
};

unsigned int UDFFindFile (dvd_reader_t *, const char *, unsigned int *);

gchar *ogmdvd_languages[][3] = 
{
  { "  ", "und", "Undetermined" },
  { "??", "und", "Undetermined" },
  { "ab", "abk", "Abkhazian" },
  { "aa", "aar", "Afar" },
  { "af", "afr", "Afrikaans" },
  { "ak", "aka", "Akan" },
  { "sq", "alb", "Albanian" },
  { "am", "amh", "Amharic" },
  { "ar", "ara", "Arabic" },
  { "an", "arg", "Aragonese" },
  { "hy", "arm", "Armenian" },
  { "as", "asm", "Assamese" },
  { "av", "ava", "Avaric" },
  { "ae", "ave", "Avestan" },
  { "ay", "aym", "Aymara" },
  { "az", "aze", "Azerbaijani" },
  { "bm", "bam", "Bambara" },
  { "ba", "bak", "Bashkir" },
  { "eu", "baq", "Basque" },
  { "be", "bel", "Belarusian" },
  { "bn", "ben", "Bengali" },
  { "bh", "bih", "Bihari" },
  { "bi", "bis", "Bislama" },
  { "bs", "bos", "Bosnian" },
  { "br", "bre", "Breton" },
  { "bg", "bul", "Bulgarian" },
  { "my", "bur", "Burmese" },
  { "ca", "cat", "Catalan" },
  { "ch", "cha", "Chamorro" },
  { "ce", "che", "Chechen" },
  { "ny", "nya", "Chewa; Chichewa; Nyanja" },
  { "zh", "chi", "Chinese" },
  { "za", "zha", "Chuang; Zhuang" },
  { "cu", "chu", "Church Slavonic; Old Bulgarian; Church Slavic;" },
  { "cv", "chv", "Chuvash" },
  { "kw", "cor", "Cornish" },
  { "co", "cos", "Corsican" },
  { "cr", "cre", "Cree" },
  { "hr", "scr", "Croatian" },
  { "cs", "cze", "Czech" },
  { "da", "dan", "Danish" },
  { "de", "ger", "Deutsch" },
  { "dv", "div", "Divehi" },
  { "dz", "dzo", "Dzongkha" },
  { "en", "eng", "English" },
  { "es", "spa", "Espanol" },
  { "eo", "epo", "Esperanto" },
  { "et", "est", "Estonian" },
  { "ee", "ewe", "Ewe" },
  { "fo", "fao", "Faroese" },
  { "fj", "fij", "Fijian" },
  { "fi", "fin", "Finnish" },
  { "fr", "fre", "Francais" },
  { "fy", "fry", "Frisian" },
  { "ff", "ful", "Fulah" },
  { "gd", "gla", "Gaelic; Scottish Gaelic" },
  { "gl", "glg", "Gallegan" },
  { "lg", "lug", "Ganda" },
  { "ka", "geo", "Georgian" },
  { "ki", "kik", "Gikuyu; Kikuyu" },
  { "el", "ell", "Greek" },
  { "kl", "kal", "Greenlandic; Kalaallisut" },
  { "gn", "grn", "Guarani" },
  { "gu", "guj", "Gujarati" },
  { "ha", "hau", "Hausa" },
  { "he", "heb", "Hebrew" },
  { "hz", "her", "Herero" },
  { "hi", "hin", "Hindi" },
  { "ho", "hmo", "Hiri Motu" },
  { "hu", "hun", "Hungarian" },
  { "is", "ice", "Icelandic" },
  { "io", "ido", "Ido" },
  { "ig", "ibo", "Igbo" },
  { "id", "ind", "Indonesian" },
  { "ia", "ina", "Interlingua (International)" },
  { "ie", "ile", "Interlingue" },
  { "iu", "iku", "Inuktitut" },
  { "ik", "ipk", "Inupiaq" },
  { "ga", "gle", "Irish" },
  { "it", "ita", "Italiano" },
  { "ja", "jpn", "Japanese" },
  { "jv", "jav", "Javanese" },
  { "kn", "kan", "Kannada" },
  { "kr", "kau", "Kanuri" },
  { "ks", "kas", "Kashmiri" },
  { "kk", "kaz", "Kazakh" },
  { "km", "khm", "Khmer" },
  { "rw", "kin", "Kinyarwanda" },
  { "ky", "kir", "Kirghiz" },
  { "kv", "kom", "Komi" },
  { "kg", "kon", "Kongo" },
  { "ko", "kor", "Korean" },
  { "ku", "kur", "Kurdish" },
  { "kj", "kua", "Kwanyama, Kuanyama" },
  { "lo", "lao", "Lao" },
  { "la", "lat", "Latin" },
  { "lv", "lav", "Latvian" },
  { "li", "lim", "Limburgish; Limburger; Limburgan" },
  { "ln", "lin", "Lingala" },
  { "lt", "lit", "Lithuanian" },
  { "lu", "lub", "Luba-Katanga" },
  { "lb", "ltz", "Luxembourgish; Letzeburgesch" },
  { "mk", "mac", "Macedonian" },
  { "mg", "mlg", "Malagasy" },
  { "ml", "mal", "Malayalam" },
  { "ms", "may", "Malay" },
  { "mt", "mlt", "Maltese" },
  { "gv", "glv", "Manx" },
  { "mi", "mao", "Maori" },
  { "mr", "mar", "Marathi" },
  { "mh", "mah", "Marshallese" },
  { "mo", "mol", "Moldavian" },
  { "mn", "mon", "Mongolian" },
  { "na", "nau", "Nauru" },
  { "nv", "nav", "Navajo; Navaho" },
  { "ng", "ndo", "Ndonga" },
  { "nl", "nld", "Nederlands" },
  { "ne", "nep", "Nepali" },
  { "se", "sme", "Northern Sami" },
  { "nd", "nde", "North Ndebele" },
  { "nb", "nob", "Norwegian Bokmal; Bokmal, Norwegian" },
  { "no", "nor", "Norwegian" },
  { "nn", "nno", "Norwegian Nynorsk; Nynorsk, Norwegian" },
  { "oj", "oji", "Ojibwa" },
  { "or", "ori", "Oriya" },
  { "om", "orm", "Oromo" },
  { "os", "oss", "Ossetian; Ossetic" },
  { "pi", "pli", "Pali" },
  { "pa", "pan", "Panjabi" },
  { "fa", "per", "Persian" },
  { "pl", "pol", "Polish" },
  { "pt", "por", "Portuguese" },
  { "oc", "oci", "Provencal; Occitan (post 1500)" },
  { "ps", "pus", "Pushto" },
  { "qu", "que", "Quechua" },
  { "rm", "roh", "Raeto-Romance" },
  { "ro", "ron", "Romanian" },
  { "rn", "run", "Rundi" },
  { "ru", "rus", "Russian" },
  { "sm", "smo", "Samoan" },
  { "sg", "sag", "Sango" },
  { "sa", "san", "Sanskrit" },
  { "sc", "srd", "Sardinian" },
  { "sr", "srp", "Serbian" },
  { "sn", "sna", "Shona" },
  { "ii", "iii", "Sichuan Yi" },
  { "sd", "snd", "Sindhi" },
  { "si", "sin", "Sinhalese" },
  { "sk", "slo", "Slovak" },
  { "sl", "slv", "Slovenian" },
  { "so", "som", "Somali" },
  { "st", "sot", "Sotho, Southern" },
  { "nr", "nbl", "South Ndebele" },
  { "su", "sun", "Sundanese" },
  { "sw", "swa", "Swahili" },
  { "ss", "ssw", "Swati" },
  { "sv", "swe", "Swedish" },
  { "tl", "tgl", "Tagalog" },
  { "ty", "tah", "Tahitian" },
  { "tg", "tgk", "Tajik" },
  { "ta", "tam", "Tamil" },
  { "tt", "tat", "Tatar" },
  { "te", "tel", "Telugu" },
  { "th", "tha", "Thai" },
  { "bo", "tib", "Tibetan" },
  { "ti", "tir", "Tigrinya" },
  { "to", "ton", "Tonga (Tonga Islands)" },
  { "ts", "tso", "Tsonga" },
  { "tn", "tsn", "Tswana" },
  { "tr", "tur", "Turkish" },
  { "tk", "tuk", "Turkmen" },
  { "tw", "twi", "Twi" },
  { "ug", "uig", "Uighur" },
  { "uk", "ukr", "Ukrainian" },
  { "ur", "urd", "Urdu" },
  { "uz", "uzb", "Uzbek" },
  { "ve", "ven", "Venda" },
  { "vi", "vie", "Vietnamese" },
  { "vo", "vol", "Volapük" },
  { "wa", "wln", "Walloon" },
  { "cy", "wel", "Welsh" },
  { "wo", "wol", "Wolof" },
  { "xh", "xho", "Xhosa" },
  { "yi", "yid", "Yiddish" },
  { "yo", "yor", "Yoruba" },
  { "zu", "zul", "Zulu" },
  { NULL, NULL,  NULL },
};

guint ogmdvd_nlanguages = sizeof (ogmdvd_languages) / sizeof (ogmdvd_languages[0]) - 1;

static glong
ogmdvd_time_to_msec (dvd_time_t *dtime)
{
  guint hour, min, sec, frames;
  gfloat fps;

  hour   = ((dtime->hour    & 0xf0) >> 4) * 10 + (dtime->hour    & 0x0f);
  min    = ((dtime->minute  & 0xf0) >> 4) * 10 + (dtime->minute  & 0x0f);
  sec    = ((dtime->second  & 0xf0) >> 4) * 10 + (dtime->second  & 0x0f);
  frames = ((dtime->frame_u & 0x30) >> 4) * 10 + (dtime->frame_u & 0x0f);

  if (((dtime->frame_u & 0xc0) >> 6) == 1)
    fps = 25.0;
  else
    fps = 30000 / 1001.0;

  return hour * 60 * 60 * 1000 + min * 60 * 1000 + sec * 1000 + (gfloat) frames * 1000.0 / fps;
}

OGMDvd *
ogmdvd_open (const gchar *device)
{
  OGMDvd *dvd;

  dvd_reader_t *reader;
  ifo_handle_t *vmg_file;

  if (!device || !*device)
    device = "/dev/dvd";

  reader = DVDOpen (device);
  if (!reader)
    return NULL;

  vmg_file = ifoOpen (reader, 0);
  if (!vmg_file)
  {
    DVDClose (reader);
    return NULL;
  }

  dvd = g_new0 (OGMDvd, 1);
  dvd->device = g_strdup (device);
  dvd->vmg_file = vmg_file;
  dvd->reader = reader;
  dvd->ref = 1;

  return dvd;
}

void
ogmdvd_ref (OGMDvd *dvd)
{
  g_return_if_fail (dvd != NULL);

  dvd->ref ++;
}

void
ogmdvd_unref (OGMDvd *dvd)
{
  g_return_if_fail (dvd != NULL);

  if (dvd->ref > 0)
  {
    dvd->ref --;
    if (dvd->ref == 0)
    {
      g_free (dvd->device);

      if (dvd->vmg_file)
        ifoClose (dvd->vmg_file);

      if (dvd->reader)
        DVDClose (dvd->reader);

      g_free (dvd);
    }
  }
}

gchar *
ogmdvd_get_title (OGMDvd *dvd)
{
  gchar title[32];

  g_return_val_if_fail (dvd != NULL, NULL);

  if (DVDUDFVolumeInfo (dvd->reader, title, 32, NULL, 0) < 0)
    return g_strdup ("Unknown");

  return g_strdup (title);
}

G_CONST_RETURN gchar *
ogmdvd_get_device (OGMDvd *dvd)
{
  g_return_val_if_fail (dvd != NULL, NULL);

  return dvd->device;
}

gint
ogmdvd_get_num_video_streams (OGMDvd *dvd)
{
  g_return_val_if_fail (dvd != NULL, -1);

  return dvd->vmg_file->tt_srpt->nr_of_srpts;
}

OGMDvdVideoStream *
ogmdvd_get_video_stream (OGMDvd *dvd, guint vid)
{
  g_return_val_if_fail (dvd != NULL, NULL);
  g_return_val_if_fail (vid < dvd->vmg_file->tt_srpt->nr_of_srpts, NULL);

  if (vid < dvd->vmg_file->tt_srpt->nr_of_srpts)
  {
    ifo_handle_t *vts_file;

    vts_file = ifoOpen (dvd->reader, dvd->vmg_file->tt_srpt->title[vid].title_set_nr);
    if (vts_file)
    {
      OGMDvdVideoStream *video;

      ogmdvd_ref (dvd);

      video = g_new0 (OGMDvdVideoStream, 1);
      video->vts_file = vts_file;
      video->dvd = dvd;
      video->vid = vid;
      video->ref = 1;

      video->ttn = dvd->vmg_file->tt_srpt->title[vid].vts_ttn;
      video->pgcn = video->vts_file->vts_ptt_srpt->title[video->ttn - 1].ptt[0].pgcn;

      return video;
    }
  }
  return NULL;
}

void
ogmdvd_video_stream_ref (OGMDvdVideoStream *video)
{
  g_return_if_fail (video != NULL);

  video->ref ++;
}

void
ogmdvd_video_stream_unref (OGMDvdVideoStream *video)
{
  g_return_if_fail (video != NULL);

  if (video->ref > 0)
  {
    video->ref --;

    if (video->ref == 0)
    {
      if (video->vts_file)
        ifoClose (video->vts_file);

      ogmdvd_unref (video->dvd);

      g_free (video);
    }
  }
}

OGMDvd *
ogmdvd_video_stream_get_dvd (OGMDvdVideoStream *video)
{
  g_return_val_if_fail (video != NULL, NULL);

  return video->dvd;
}

gint
ogmdvd_video_stream_get_id (OGMDvdVideoStream *video)
{
  g_return_val_if_fail (video != NULL, -1);

  return video->vid;
}

void
ogmdvd_video_stream_get_raw_size (OGMDvdVideoStream *video, guint *width, guint *height)
{
  g_return_if_fail (video != NULL);
  g_return_if_fail (width != NULL);
  g_return_if_fail (height != NULL);

  *width = 0;
  *height = 480;
  if (video->vts_file->vtsi_mat->vts_video_attr.video_format != 0)
    *height = 576;

  switch (video->vts_file->vtsi_mat->vts_video_attr.picture_size)
  {
    case 0:
      *width = 720;
      break;
    case 1:
      *width = 704;
      break;
    case 2:
      *width = 352;
      break;
    case 3:
      *width = 352;
      *width /= 2;
      break;
    default:
      g_assert_not_reached ();
      break;
  }
}

glong
ogmdvd_video_stream_get_length (OGMDvdVideoStream *video, OGMDvdTime  *time_)
{
  dvd_time_t *dtime;
  pgc_t *pgc;

  gfloat fps;
  guint dhour, dmin, dsec, dframes;

  g_return_val_if_fail (video != NULL, -1);

  pgc = video->vts_file->vts_pgcit->pgci_srp[video->pgcn - 1].pgc;

  dtime = &pgc->playback_time;

  dhour   = ((dtime->hour    & 0xf0) >> 4) * 10 + (dtime->hour    & 0x0f);
  dmin    = ((dtime->minute  & 0xf0) >> 4) * 10 + (dtime->minute  & 0x0f);
  dsec    = ((dtime->second  & 0xf0) >> 4) * 10 + (dtime->second  & 0x0f);
  dframes = ((dtime->frame_u & 0x30) >> 4) * 10 + (dtime->frame_u & 0x0f);

  if (((dtime->frame_u & 0xc0) >> 6) == 1)
    fps = 25.0;
  else
    fps = 30000 / 1001.0;

  if (time_)
  {
    time_->hour = dhour;
    time_->min = dmin;
    time_->sec = dsec;
    time_->frames = dframes;
  }

  return (glong) (dhour * 3600 * fps + dmin * 60 * fps + dsec * fps + dframes);
}

void
ogmdvd_video_stream_get_framerate (OGMDvdVideoStream *video, guint *numerator, guint *denominator)
{
  pgc_t *pgc;

  g_return_if_fail (video != NULL);
  g_return_if_fail (numerator != NULL);
  g_return_if_fail (denominator != NULL);

  pgc = video->vts_file->vts_pgcit->pgci_srp[video->pgcn - 1].pgc;

  switch ((pgc->playback_time.frame_u & 0xc0) >> 6)
  {
    case 1:
      *numerator = 25;
      *denominator = 1;
      break;
    case 3:
      *numerator = 30000;
      *denominator = 1001;
      break;
    default:
      g_assert_not_reached ();
      break;
  }
}

void
ogmdvd_video_stream_get_aspect_ratio  (OGMDvdVideoStream *video, guint *numerator, guint *denominator)
{
  g_return_if_fail (video != NULL);
  g_return_if_fail (numerator != NULL);
  g_return_if_fail (denominator != NULL);

  switch (video->vts_file->vtsi_mat->vts_video_attr.display_aspect_ratio)
  {
    case 0:
      *numerator = 4;
      *denominator = 3;
      break;
    case 3:
      *numerator = 16;
      *denominator = 9;
      break;
    default:
      g_assert_not_reached ();
      break;
  }
}

gint
ogmdvd_video_stream_get_display_aspect (OGMDvdVideoStream *video)
{
  gint aspect;

  g_return_val_if_fail (video != NULL, -1);

  aspect = video->vts_file->vtsi_mat->vts_video_attr.display_aspect_ratio;

  return aspect == 3 ? OGMDVD_VIDEO_ASPECT_16_9 : aspect;
}

gint
ogmdvd_video_stream_get_format (OGMDvdVideoStream *video)
{
  g_return_val_if_fail (video != NULL, -1);

  return video->vts_file->vtsi_mat->vts_video_attr.video_format;
}

gint
ogmdvd_video_stream_get_display_format (OGMDvdVideoStream *video)
{
  g_return_val_if_fail (video != NULL, -1);

  return video->vts_file->vtsi_mat->vts_video_attr.permitted_df;
}

gint
ogmdvd_video_stream_get_num_audio_streams (OGMDvdVideoStream *video)
{
  gint i, naudio;

  g_return_val_if_fail (video != NULL, -1);

  for (naudio = i = 0; i < video->vts_file->vtsi_mat->nr_of_vts_audio_streams; i++)
    if (video->vts_file->vts_pgcit->pgci_srp[0].pgc->audio_control[i] & 0x8000)
      naudio ++;

  return naudio;
}

OGMDvdAudioStream *
ogmdvd_video_stream_get_audio_stream (OGMDvdVideoStream *video, guint aid)
{
  OGMDvdAudioStream *audio;

  g_return_val_if_fail (video != NULL, NULL);
  g_return_val_if_fail (aid < video->vts_file->vtsi_mat->nr_of_vts_audio_streams, NULL);
  g_return_val_if_fail (video->vts_file->vts_pgcit->pgci_srp[0].pgc->audio_control[aid] & 0x8000, NULL);

  ogmdvd_video_stream_ref (video);

  audio = g_new0 (OGMDvdAudioStream, 1);
  audio->attr = &video->vts_file->vtsi_mat->vts_audio_attr[aid];
  audio->video = video;
  audio->aid = aid;
  audio->ref = 1;

  return audio;
}

void
ogmdvd_video_stream_get_palette (OGMDvdVideoStream *vstream, guint *palette)
{
  guint i;
  pgc_t *pgc;

  g_return_if_fail (vstream != NULL);
  g_return_if_fail (palette != NULL);

  pgc = vstream->vts_file->vts_pgcit->pgci_srp[vstream->pgcn - 1].pgc;

  for (i = 0; i < 16; i++)
    palette[i] = pgc->palette[i];
}

static gint64
ogmdvd_get_ifo_size (OGMDvd *dvd, guint vts)
{
  gchar filename[FILENAME_MAX];
  guint size;

  if (vts == 0)
    strncpy (filename, "/VIDEO_TS/VIDEO_TS.IFO", FILENAME_MAX);
  else
    snprintf (filename, FILENAME_MAX, "/VIDEO_TS/VTS_%02u_0.IFO", vts);

  if (!UDFFindFile (dvd->reader, filename, &size))
    return -1;

  return size;
}

static gint64
ogmdvd_get_bup_size (OGMDvd *dvd, guint vts)
{
  gchar filename[FILENAME_MAX];
  guint size;

  if (vts == 0)
    strncpy (filename, "/VIDEO_TS/VIDEO_TS.BUP", FILENAME_MAX);
  else
    snprintf (filename, FILENAME_MAX, "/VIDEO_TS/VTS_%02u_0.BUP", vts);

  if (!UDFFindFile (dvd->reader, filename, &size))
    return -1;

  return size;
}

static gint64
ogmdvd_get_menu_size (OGMDvd *dvd, guint vts)
{
  gchar filename[FILENAME_MAX];
  guint size;

  if (vts == 0)
    strncpy (filename, "/VIDEO_TS/VIDEO_TS.VOB", FILENAME_MAX);
  else
    snprintf (filename, FILENAME_MAX, "/VIDEO_TS/VTS_%02u_0.VOB", vts);

  if (!UDFFindFile (dvd->reader, filename, &size))
    return -1;

  return size;
}

static gint64
ogmdvd_get_vob_size (OGMDvd *dvd, guint vts)
{
  gchar filename[FILENAME_MAX];
  guint vob, size;
  gint64 fullsize;

  if (vts == 0)
    return 0;

  vob = 1; fullsize = 0;
  while (1)
  {
    snprintf (filename, FILENAME_MAX, "/VIDEO_TS/VTS_%02u_%u.VOB", vts, vob++);
    if (!UDFFindFile (dvd->reader, filename, &size))
      break;
    fullsize += size;
  }

  if (vob == 1)
    return -1;

  return fullsize;
}

gint64
ogmdvd_get_vmg_size (OGMDvd *dvd)
{
  gint64 size, fullsize = 0;

  g_return_val_if_fail (dvd != NULL, -1);

  if ((size = ogmdvd_get_ifo_size (dvd, 0)) < 0)
    return -1;
  fullsize += size;

  if ((size = ogmdvd_get_bup_size (dvd, 0)) < 0)
    return -1;
  fullsize += size;

  if ((size = ogmdvd_get_menu_size (dvd, 0)) < 0)
    return -1;
  fullsize += size;

  return fullsize;
}

gint64
ogmdvd_video_stream_get_vts_size (OGMDvdVideoStream *video)
{
  OGMDvd *dvd;
  gint64 size, fullsize = 0;
  guint vts;

  g_return_val_if_fail (video != NULL, -1);

  dvd = ogmdvd_video_stream_get_dvd (video);

  vts = dvd->vmg_file->tt_srpt->title[video->vid].title_set_nr;

  if ((size = ogmdvd_get_ifo_size (dvd, vts)) < 0)
    return -1;
  fullsize += size;

  if ((size = ogmdvd_get_bup_size (dvd, vts)) < 0)
    return -1;
  fullsize += size;

  if ((size = ogmdvd_get_menu_size (dvd, vts)) < 0)
    return -1;
  fullsize += size;

  if (vts > 0)
  {
    if ((size = ogmdvd_get_vob_size (dvd, vts)) < 0)
      return -1;
    fullsize += size;
  }

  return fullsize;
}

void
ogmdvd_audio_stream_ref (OGMDvdAudioStream *audio)
{
  g_return_if_fail (audio != NULL);

  audio->ref ++;
}

void
ogmdvd_audio_stream_unref (OGMDvdAudioStream *audio)
{
  g_return_if_fail (audio != NULL);

  if (audio->ref > 0)
  {
    audio->ref --;

    if (audio->ref == 0)
    {
      ogmdvd_video_stream_unref (audio->video);
      g_free (audio);
    }
  }
}

OGMDvdVideoStream *
ogmdvd_audio_stream_get_video_stream (OGMDvdAudioStream *audio)
{
  g_return_val_if_fail (audio != NULL, NULL);

  return audio->video;
}

gint
ogmdvd_audio_stream_get_id (OGMDvdAudioStream *audio)
{
  g_return_val_if_fail (audio != NULL, -1);

  return audio->aid;
}

gint
ogmdvd_audio_stream_get_format (OGMDvdAudioStream *audio)
{
  g_return_val_if_fail (audio != NULL, -1);

  return audio->attr->audio_format;
}

gint
ogmdvd_audio_stream_get_channels (OGMDvdAudioStream *audio)
{
  g_return_val_if_fail (audio != NULL, -1);

  return audio->attr->channels;
}

gint
ogmdvd_audio_stream_get_language (OGMDvdAudioStream *audio)
{
  g_return_val_if_fail (audio != NULL, -1);

  if (audio->attr->lang_type == 1)
    return audio->attr->lang_code;

  return -1;
}

gint
ogmdvd_audio_stream_get_quantization (OGMDvdAudioStream *audio)
{
  g_return_val_if_fail (audio != NULL, -1);

  return audio->attr->quantization;
}

gint
ogmdvd_audio_stream_get_type (OGMDvdAudioStream *audio)
{
  g_return_val_if_fail (audio != NULL, -1);

  return audio->attr->code_extension;
}

gint
ogmdvd_video_stream_get_num_chapters (OGMDvdVideoStream *video)
{
  pgc_t *pgc;

  g_return_val_if_fail (video != NULL, -1);

  pgc = video->vts_file->vts_pgcit->pgci_srp[video->pgcn - 1].pgc;

  return pgc->nr_of_programs;
}

gint
ogmdvd_video_stream_get_num_angles (OGMDvdVideoStream *video)
{
  g_return_val_if_fail (video != NULL, -1);

  return video->dvd->vmg_file->tt_srpt->title[video->vid].nr_of_angles;
}

glong
ogmdvd_video_stream_get_chapters_length (OGMDvdVideoStream *video, OGMDvdTime *time_, guint start, gint end)
{
  pgc_t *pgc;
  guint pgn, pgcn;
  glong last, cell, total;
  gfloat fps;

  g_return_val_if_fail (video != NULL, -1);
  g_return_val_if_fail (start <= end, -1);

  pgc = video->vts_file->vts_pgcit->pgci_srp[video->pgcn - 1].pgc;

  g_return_val_if_fail (start < pgc->nr_of_programs, -1);

  pgcn = video->vts_file->vts_ptt_srpt->title[video->ttn - 1].ptt[start].pgcn;
  pgc = video->vts_file->vts_pgcit->pgci_srp[pgcn - 1].pgc;

  pgn = video->vts_file->vts_ptt_srpt->title[video->ttn - 1].ptt[start].pgn;
  cell = pgc->program_map[pgn - 1] - 1;

  last = pgc->nr_of_cells;
  if (end > -1 && end < pgc->nr_of_programs - 1)
  {
    pgn = video->vts_file->vts_ptt_srpt->title[video->ttn - 1].ptt[end + 1].pgn;
    last = pgc->program_map[pgn - 1] - 1;
  }

  for (total = 0; cell < last; cell ++)
    total += ogmdvd_time_to_msec (&pgc->cell_playback[cell].playback_time);

  if (time_)
  {
    time_->hour = total / (60 * 60 * 1000);
    time_->min = total / (60 * 1000) % 60;
    time_->sec = total / 1000 % 60;
    time_->frames = total % 1000;
  }

  if ((video->vts_file->vts_pgcit->pgci_srp[video->pgcn - 1].pgc->playback_time.frame_u & 0xc0) >> 6)
    fps = 25.0;
  else
    fps = 30000 / 1001.0;

  return (glong) (total * fps / 1000.0);
}

gint
ogmdvd_video_stream_get_num_subp_streams (OGMDvdVideoStream *video)
{
  gint i, nsubp;

  g_return_val_if_fail (video != NULL, -1);

  for (nsubp = i = 0; i < video->vts_file->vtsi_mat->nr_of_vts_subp_streams; i++)
    if (video->vts_file->vts_pgcit->pgci_srp[0].pgc->subp_control[i] & 0x80000000)
      nsubp ++;

  return nsubp;
}

OGMDvdSubpStream *
ogmdvd_video_stream_get_subp_stream (OGMDvdVideoStream *video, guint sid)
{
  OGMDvdSubpStream *subp;

  g_return_val_if_fail (video != NULL, NULL);
  g_return_val_if_fail (sid < video->vts_file->vtsi_mat->nr_of_vts_subp_streams, NULL);
  g_return_val_if_fail (video->vts_file->vts_pgcit->pgci_srp[0].pgc->subp_control[sid] & 0x80000000, NULL);

  ogmdvd_video_stream_ref (video);

  subp = g_new0 (OGMDvdSubpStream, 1);
  subp->attr = &video->vts_file->vtsi_mat->vts_subp_attr[sid];
  subp->video = video;
  subp->sid = sid;
  subp->ref = 1;

  return subp;
}

void
ogmdvd_subp_stream_ref (OGMDvdSubpStream *subp)
{
  g_return_if_fail (subp != NULL);

  subp->ref ++;
}

void
ogmdvd_subp_stream_unref (OGMDvdSubpStream *subp)
{
  g_return_if_fail (subp != NULL);

  if (subp->ref > 0)
  {
    subp->ref --;

    if (subp->ref == 0)
    {
      ogmdvd_video_stream_unref (subp->video);
      g_free (subp);
    }
  }
}

OGMDvdVideoStream *
ogmdvd_subp_stream_get_video_stream (OGMDvdSubpStream *subp)
{
  g_return_val_if_fail (subp != NULL, NULL);

  return subp->video;
}

gint
ogmdvd_subp_stream_get_id (OGMDvdSubpStream *subp)
{
  g_return_val_if_fail (subp != NULL, -1);

  return subp->sid;
}

gint
ogmdvd_subp_stream_get_language (OGMDvdSubpStream *subp)
{
  g_return_val_if_fail (subp != NULL, -1);

  return subp->attr->lang_code;
}

gint
ogmdvd_subp_stream_get_type (OGMDvdSubpStream *subp)
{
  g_return_val_if_fail (subp != NULL, -1);

  return subp->attr->lang_extension;
}

G_CONST_RETURN gchar *
ogmdvd_get_language_iso639_1 (gint code)
{
  static gchar lang[3];

  if (code < 1)
  {
    lang[0] = '?';
    lang[1] = '?';
    lang[2] = 0;
  }
  else
  {
    lang[0] = code >> 8;
    lang[1] = code & 0xff;
    lang[2] = 0;
  }

  return lang;
}

G_CONST_RETURN gchar *
ogmdvd_get_language_iso639_2 (gint code)
{
  const gchar *lang;
  guint i;

  lang = ogmdvd_get_language_iso639_1 (code);

  for (i = 0; ogmdvd_languages[i][OGMDVD_LANGUAGE_ISO639_1]; i++)
    if (strcmp (ogmdvd_languages[i][OGMDVD_LANGUAGE_ISO639_1], lang) == 0)
      return ogmdvd_languages[i][OGMDVD_LANGUAGE_ISO639_2];

  return NULL;
}

G_CONST_RETURN gchar *
ogmdvd_get_language_name (gint code)
{
  const gchar *lang;
  guint i;

  lang = ogmdvd_get_language_iso639_1 (code);

  for (i = 0; ogmdvd_languages[i][OGMDVD_LANGUAGE_ISO639_1]; i++)
    if (strcmp (ogmdvd_languages[i][OGMDVD_LANGUAGE_ISO639_1], lang) == 0)
      return ogmdvd_languages[i][OGMDVD_LANGUAGE_NAME];

  return NULL;
}

G_CONST_RETURN gchar *
ogmdvd_get_video_format_name (gint format)
{
  static gchar *video_format[] = 
  {
    "NTSC",
    "PAL",
    "Error",
    "Error"
  };

  return format < 0 ? NULL : video_format[format];
}

G_CONST_RETURN gchar *
ogmdvd_get_video_aspect_name (gint aspect)
{
  static gchar *video_aspect[] =
  {
    "4/3", 
    "16/9", 
    "?:?", 
    "16/9"
  };

  return video_aspect[aspect];
}

G_CONST_RETURN gchar *
ogmdvd_get_audio_type_name (gint type)
{
  static gchar *audio_type[] = 
  {
    "Undefined", 
    "Normal", 
    "Impaired", 
    "Comments 1", 
    "Comments 2"
  };

  return type < 0 ? NULL : audio_type[type];
}

G_CONST_RETURN gchar *
ogmdvd_get_audio_format_name (gint format)
{
  static gchar *audio_format[] =
  {
    "AC3",
    "Unknown",
    "MPEG1",
    "MPEG2EXT",
    "LPCM",
    "SDDS",
    "DTS",
    "Error"
  };

  return format < 0 ? NULL : audio_format[format];
}

G_CONST_RETURN gchar *
ogmdvd_get_audio_channels_name (gint channels)
{
  static gchar *audio_channels[] =
  {
    "Mono",
    "Stereo",
    "Unknown",
    "Surround",
    "Unknown",
    "5.1",
    "6.1",
    "7.1"
  };

  return channels < 0 ? NULL : audio_channels[channels];
}

G_CONST_RETURN gchar *
ogmdvd_get_subp_type_name (gint type)
{
  static gchar *subp_type[] = 
  {
    "Undefined", 
    "Normal", 
    "Large", 
    "Children", 
    "Reserved", 
    "Normal_CC", 
    "Large_CC", 
    "Children_CC",
    "Reserved", 
    "Forced", 
    "Reserved", 
    "Reserved", 
    "Reserved", 
    "Director", 
    "Large_Director", 
    "Children_Director"
  };

  return type < 0 ? NULL : subp_type[type];
}

G_CONST_RETURN gchar *
ogmdvd_get_audio_quantization_name (gint quantization)
{
  static gchar *audio_quantization[] = 
  {
    "16bit", 
    "20bit", 
    "24bit", 
    "drc"
  };

  return audio_quantization[quantization];
}

