/* OGMRip - A library for DVD ripping and encoding
 * Copyright (C) 2004-2005 Olivier Rolland <billl@esil.univ-mrs.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "ogmrip-backend.h"
#include "ogmsp-enums.h"
#include "ogmsp-exec.h"
#include "ogmrip-ogg.h"

#include <unistd.h>
#include <math.h>

#include <glib/gstdio.h>

static gint   ogmrip_ogg_run               (OGMSpSpawn      *spawn);
static gint64 ogmrip_ogg_get_overhead_size (OGMRipContainer *container,
                                            guint64         size,
                                            gulong          length);

G_DEFINE_TYPE (OGMRipOgg, ogmrip_ogg, OGMRIP_TYPE_CONTAINER)

static void
ogmrip_ogg_class_init (OGMRipOggClass *klass)
{
  OGMSpSpawnClass *spawn_class;
  OGMRipContainerClass *container_class;

  spawn_class = OGMSP_SPAWN_CLASS (klass);
  container_class = OGMRIP_CONTAINER_CLASS (klass);

  spawn_class->run = ogmrip_ogg_run;
  container_class->get_overhead_size = ogmrip_ogg_get_overhead_size;
}

static void
ogmrip_ogg_init (OGMRipOgg *ogg)
{
}

static gint
ogmrip_ogg_run (OGMSpSpawn *spawn)
{
  OGMSpSpawn *child;
  gint result, tnumber, fd;
  gchar **argv, *file;

  fd = 0;
  file = NULL;
  result = OGMSP_RESULT_ERROR;

  tnumber = ogmrip_container_get_tnumber (OGMRIP_CONTAINER (spawn));
  if (tnumber > 1)
  {
    gchar *dirname;

    dirname = g_path_get_dirname (ogmrip_container_get_output (OGMRIP_CONTAINER (spawn)));
    file = g_build_filename (dirname, "merge.XXXXXX", NULL);
    g_free (dirname);

    fd = g_mkstemp (file);
    if (fd < 0)
    {
      g_free (file);
      return OGMSP_RESULT_ERROR;
    }
  }

  argv = ogmrip_backend_ogg_merge_command (OGMRIP_CONTAINER (spawn), file);
  if (argv)
  {
    child = ogmsp_exec_newv (argv);
    ogmsp_exec_add_watch_full (OGMSP_EXEC (child), (OGMSpWatch) ogmrip_backend_ogg_merge_watch, spawn, TRUE, FALSE, FALSE);
    ogmsp_container_add (OGMSP_CONTAINER (spawn), child);
    g_object_unref (child);

    result = OGMSP_SPAWN_CLASS (ogmrip_ogg_parent_class)->run (spawn);

    ogmsp_container_remove (OGMSP_CONTAINER (spawn), child);
  }

  if (tnumber > 1 && result == OGMSP_RESULT_COMPLETED)
  {
    argv = ogmrip_backend_ogg_split_command (OGMRIP_CONTAINER (spawn), file);
    if (argv)
    {
      child = ogmsp_exec_newv (argv);
      ogmsp_exec_add_watch_full (OGMSP_EXEC (child), (OGMSpWatch) ogmrip_backend_ogg_split_watch, spawn, TRUE, FALSE, FALSE);
      ogmsp_container_add (OGMSP_CONTAINER (spawn), child);
      g_object_unref (child);

      result = OGMSP_SPAWN_CLASS (ogmrip_ogg_parent_class)->run (spawn);

      ogmsp_container_remove (OGMSP_CONTAINER (spawn), child);
    }
  }

  if (file)
  {
    g_unlink (file);
    g_free (file);
  }

  if (fd)
    close (fd);

  return result;
}

OGMSpSpawn *
ogmrip_ogg_new (const gchar *output)
{
  OGMRipOgg *ogg;

  g_return_val_if_fail (output && *output, NULL);

  ogg = g_object_new (OGMRIP_TYPE_OGG, NULL);
  ogmrip_container_set_output (OGMRIP_CONTAINER (ogg), output);

  return OGMSP_SPAWN (ogg);
}

static gint64
ogmrip_ogg_get_overhead_size (OGMRipContainer *container, guint64 size, gulong length)
{
  gint64 video_size;

  g_return_val_if_fail (OGMRIP_IS_OGG (container), -1);

  video_size = size - ogmrip_container_get_nonvideo_size (container);

  return ceil (video_size - video_size / 1.0115);
}

