/* OGMRip - A library for DVD ripping and encoding
 * Copyright (C) 2004-2005 Olivier Rolland <billl@esil.univ-mrs.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "ogmrip-vobsub.h"
#include "ogmrip-backend.h"
#include "ogmrip-tools.h"

#include "ogmsp-queue.h"
#include "ogmsp-enums.h"
#include "ogmsp-exec.h"

#include <stdio.h>
#include <string.h>
#include <unistd.h>

static gint ogmrip_vobsub_run (OGMSpSpawn *spawn);

G_DEFINE_TYPE (OGMRipVobSub, ogmrip_vobsub, OGMRIP_TYPE_SUBP)

static void
ogmrip_vobsub_class_init (OGMRipVobSubClass *klass)
{
  OGMSpSpawnClass *spawn_class;

  spawn_class = OGMSP_SPAWN_CLASS (klass);

  spawn_class->run = ogmrip_vobsub_run;
}

static void
ogmrip_vobsub_init (OGMRipVobSub *vobsub)
{
}

static gint
ogmrip_vobsub_run (OGMSpSpawn *spawn)
{
  OGMSpSpawn *child;
  gchar **argv;
  gint result;

  argv = ogmrip_backend_vobsub_command (OGMRIP_SUBP (spawn), NULL, NULL);
  if (!argv)
    return OGMSP_RESULT_ERROR;

  child = ogmsp_exec_newv (argv);
  ogmsp_exec_add_watch_full (OGMSP_EXEC (child), (OGMSpWatch) ogmrip_backend_vobsub_watch, spawn, TRUE, FALSE, FALSE);
  ogmsp_container_add (OGMSP_CONTAINER (spawn), child);
  g_object_unref (child);

  result = OGMSP_SPAWN_CLASS (ogmrip_vobsub_parent_class)->run (spawn);

  ogmsp_container_remove (OGMSP_CONTAINER (spawn), child);

  return result;
}

OGMSpSpawn *
ogmrip_vobsub_new (OGMDvdSubpStream *subp, const gchar *output)
{
  OGMSpSpawn *spawn;

  spawn = g_object_new (OGMRIP_TYPE_VOBSUB, NULL);
  ogmrip_codec_set_output (OGMRIP_CODEC (spawn), output);
  ogmrip_subp_set_dvd_subp_stream (OGMRIP_SUBP (spawn), subp);

  return spawn;
}

