/* OGMRip - A library for DVD ripping and encoding
 * Copyright (C) 2004-2005 Olivier Rolland <billl@esil.univ-mrs.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "ogmrip-wav.h"
#include "ogmrip-backend.h"

#include "ogmsp-enums.h"
#include "ogmsp-exec.h"

#include <stdio.h>

#define OGMRIP_WAV_GET_PRIVATE(o) \
  (G_TYPE_INSTANCE_GET_PRIVATE ((o), OGMRIP_TYPE_WAV, OGMRipWavPriv))

struct _OGMRipWavPriv
{
  gboolean header;
};

static gint ogmrip_wav_run (OGMSpSpawn *spawn);

G_DEFINE_TYPE (OGMRipWav, ogmrip_wav, OGMRIP_TYPE_AUDIO)

static void
ogmrip_wav_class_init (OGMRipWavClass *klass)
{
  OGMSpSpawnClass *spawn_class;

  spawn_class = OGMSP_SPAWN_CLASS (klass);

  spawn_class->run = ogmrip_wav_run;

  g_type_class_add_private (klass, sizeof (OGMRipWavPriv));
}

static void
ogmrip_wav_init (OGMRipWav *wav)
{
  wav->priv = OGMRIP_WAV_GET_PRIVATE (wav);
  wav->priv->header = TRUE;
}

static gint
ogmrip_wav_run (OGMSpSpawn *spawn)
{
  OGMSpSpawn *child;
  gchar **argv;
  gint result;

  argv = ogmrip_backend_wav_command (OGMRIP_AUDIO (spawn), 
      OGMRIP_WAV (spawn)->priv->header, NULL, NULL);
  if (!argv)
    return OGMSP_RESULT_ERROR;

  child = ogmsp_exec_newv (argv);
  ogmsp_exec_add_watch_full (OGMSP_EXEC (child), (OGMSpWatch) ogmrip_backend_wav_watch, spawn, TRUE, FALSE, FALSE);
  ogmsp_container_add (OGMSP_CONTAINER (spawn), child);
  g_object_unref (child);

  result = OGMSP_SPAWN_CLASS (ogmrip_wav_parent_class)->run (spawn);

  ogmsp_container_remove (OGMSP_CONTAINER (spawn), child);

  return result;
}

OGMSpSpawn *
ogmrip_wav_new (OGMDvdAudioStream *audio, const gchar *output)
{
  OGMSpSpawn *spawn;

  spawn = g_object_new (OGMRIP_TYPE_WAV, NULL);
  ogmrip_codec_set_output (OGMRIP_CODEC (spawn), output);
  ogmrip_audio_set_dvd_audio_stream (OGMRIP_AUDIO (spawn), audio);

  return spawn;
}

void
ogmrip_wav_set_header (OGMRipWav *wav, gboolean header)
{
  g_return_if_fail (OGMRIP_IS_WAV (wav));

  wav->priv->header = header;
}

gboolean
ogmrip_wav_get_header (OGMRipWav *wav)
{
  g_return_val_if_fail (OGMRIP_IS_WAV (wav), FALSE);

  return wav->priv->header;
}

