/* OGMSpawn - A library to spawn processes
 * Copyright (C) 2004-2005 Olivier Rolland <billl@esil.univ-mrs.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "ogmsp-container.h"

#define OGMSP_CONTAINER_GET_PRIVATE(o) \
  (G_TYPE_INSTANCE_GET_PRIVATE ((o), OGMSP_TYPE_CONTAINER, OGMSpContainerPriv))

enum
{
  ADD,
  REMOVE,
  LAST_SIGNAL
};

static void ogmsp_container_finalize    (GObject        *object);
static void ogmsp_container_cancel      (OGMSpSpawn     *spawn);
static void ogmsp_container_real_add    (OGMSpContainer *container, 
                                         OGMSpSpawn     *child);
static void ogmsp_container_real_remove (OGMSpContainer *container,
                                         OGMSpSpawn     *spawn);

static gint signals[LAST_SIGNAL] = { 0 };

G_DEFINE_ABSTRACT_TYPE (OGMSpContainer, ogmsp_container, OGMSP_TYPE_SPAWN)

static void
ogmsp_container_class_init (OGMSpContainerClass *klass)
{
  GObjectClass *gobject_class;
  OGMSpSpawnClass *spawn_class;

  gobject_class = G_OBJECT_CLASS (klass);
  spawn_class = OGMSP_SPAWN_CLASS (klass);

  gobject_class->finalize = ogmsp_container_finalize;

  spawn_class->cancel = ogmsp_container_cancel;

  klass->add = ogmsp_container_real_add;
  klass->remove = ogmsp_container_real_remove;

  signals[ADD] = g_signal_new ("add", 
      G_TYPE_FROM_CLASS (gobject_class), G_SIGNAL_RUN_FIRST, 
      G_STRUCT_OFFSET (OGMSpContainerClass, add), 
      NULL, NULL, g_cclosure_marshal_VOID__OBJECT, 
      G_TYPE_NONE, 1, OGMSP_TYPE_SPAWN);

  signals[REMOVE] = g_signal_new ("remove", 
      G_TYPE_FROM_CLASS (gobject_class), G_SIGNAL_RUN_FIRST, 
      G_STRUCT_OFFSET (OGMSpContainerClass, remove), 
      NULL, NULL, g_cclosure_marshal_VOID__OBJECT, 
      G_TYPE_NONE, 1, OGMSP_TYPE_SPAWN);
}

static void
ogmsp_container_init (OGMSpContainer *container)
{
}

static void
ogmsp_container_finalize (GObject *gobject)
{
  ogmsp_container_foreach (OGMSP_CONTAINER (gobject), 
      (OGMSpCallback) g_object_unref, NULL);

  G_OBJECT_CLASS (ogmsp_container_parent_class)->finalize (gobject);
}

static void
ogmsp_container_cancel (OGMSpSpawn *spawn)
{
  ogmsp_container_foreach (OGMSP_CONTAINER (spawn), 
      (OGMSpCallback) ogmsp_spawn_cancel, NULL);
}

void
ogmsp_container_real_add (OGMSpContainer *container, OGMSpSpawn *spawn)
{
  g_warning ("Not implemented\n");
}

void
ogmsp_container_real_remove (OGMSpContainer *container, OGMSpSpawn *spawn)
{
  g_warning ("Not implemented\n");
}

void
ogmsp_container_add (OGMSpContainer *container, OGMSpSpawn *spawn)
{
  g_return_if_fail (OGMSP_IS_CONTAINER (container));
  g_return_if_fail (OGMSP_IS_SPAWN (spawn));

  if (ogmsp_spawn_get_parent (spawn))
    g_warning ("Can't add a spawn which is already in a container.\n");
  else
  {
    g_signal_emit (container, signals[ADD], 0, spawn);
    ogmsp_spawn_set_parent (spawn, OGMSP_SPAWN (container));
  }
}

void
ogmsp_container_remove (OGMSpContainer *container, OGMSpSpawn *spawn)
{
  g_return_if_fail (OGMSP_IS_CONTAINER (container));
  g_return_if_fail (OGMSP_IS_SPAWN (spawn));

  ogmsp_spawn_set_parent (spawn, NULL);

  g_signal_emit (container, signals[REMOVE], 0, spawn);
}

void
ogmsp_container_foreach (OGMSpContainer *container, OGMSpCallback callback, gpointer data)
{
  OGMSpContainerClass *klass;

  g_return_if_fail (OGMSP_IS_CONTAINER (container));

  klass = OGMSP_CONTAINER_GET_CLASS (container);

  if (klass->forall)
    (* klass->forall) (container, callback, data);
}

