/* OGMSpawn - A library to spawn processes
 * Copyright (C) 2004-2005 Olivier Rolland <billl@esil.univ-mrs.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "ogmsp-list.h"

#define OGMSP_LIST_GET_PRIVATE(o) \
  (G_TYPE_INSTANCE_GET_PRIVATE ((o), OGMSP_TYPE_LIST, OGMSpListPriv))

struct _OGMSpListPriv
{
  GList *children;
};

static void ogmsp_list_finalize (GObject        *object);
static void ogmsp_list_add      (OGMSpContainer *container,
                                 OGMSpSpawn     *spawn);
static void ogmsp_list_remove   (OGMSpContainer *container,
                                 OGMSpSpawn     *spawn);
static void ogmsp_list_forall   (OGMSpContainer *container,
                                 OGMSpCallback  callback,
                                 gpointer       data);

G_DEFINE_ABSTRACT_TYPE (OGMSpList, ogmsp_list, OGMSP_TYPE_CONTAINER)

static void
ogmsp_list_class_init (OGMSpListClass *klass)
{
  GObjectClass *gobject_class;
  OGMSpContainerClass *container_class;

  gobject_class = G_OBJECT_CLASS (klass);
  container_class = OGMSP_CONTAINER_CLASS (klass);

  gobject_class->finalize = ogmsp_list_finalize;
  
  container_class->add = ogmsp_list_add;
  container_class->remove = ogmsp_list_remove;
  container_class->forall = ogmsp_list_forall;

  g_type_class_add_private (klass, sizeof (OGMSpListPriv));
}

static void
ogmsp_list_init (OGMSpList *list)
{
  list->priv = OGMSP_LIST_GET_PRIVATE (list);
  list->priv->children = NULL;
}

static void
ogmsp_list_finalize (GObject *gobject)
{
  G_OBJECT_CLASS (ogmsp_list_parent_class)->finalize (gobject);

  g_list_free (OGMSP_LIST (gobject)->priv->children);
}

static void
ogmsp_list_add (OGMSpContainer *container, OGMSpSpawn *spawn)
{
  OGMSpList *list;

  g_object_ref (spawn);

  list = OGMSP_LIST (container);
  list->priv->children = g_list_append (list->priv->children, spawn);
}

static void
ogmsp_list_remove (OGMSpContainer *container, OGMSpSpawn *spawn)
{
  OGMSpList *list;
  GList *link;

  list = OGMSP_LIST (container);
  link = g_list_find (list->priv->children, spawn);
  if (link)
  {
    g_object_unref (spawn);
    list->priv->children = g_list_remove_link (list->priv->children, link);
    g_list_free (link);
  }
}

static void
ogmsp_list_forall (OGMSpContainer *container, OGMSpCallback callback, gpointer user_data)
{
  if (callback)
    g_list_foreach (OGMSP_LIST (container)->priv->children, (GFunc) callback, user_data);
}

GList *
ogmsp_list_get_children (OGMSpList *list)
{
  g_return_val_if_fail (OGMSP_IS_LIST (list), NULL);

  return g_list_copy (list->priv->children);
}

